@@ -59,6 +59,63 @@ def _inflate_to_namedtuple(orig_cls):
     return new_cls
 
 
+@_inflate_to_namedtuple
+class IPRiskReason:
+    """Reason for the IP risk.
+
+    This class provides both a machine-readable code and a human-readable
+    explanation of the reason for the IP risk score.
+
+    Although more codes may be added in the future, the current codes are:
+
+    - ``ANONYMOUS_IP`` - The IP address belongs to an anonymous network. See
+      the object at ``->ipAddress->traits`` for more details.
+    - ``BILLING_POSTAL_VELOCITY`` - Many different billing postal codes have
+      been seen on this IP address.
+    - ``EMAIL_VELOCITY`` - Many different email addresses have been seen on this
+      IP address.
+    - ``HIGH_RISK_DEVICE`` - A high risk device was seen on this IP address.
+    - ``HIGH_RISK_EMAIL`` - A high risk email address was seen on this IP
+      address in your past transactions.
+    - ``ISSUER_ID_NUMBER_VELOCITY`` - Many different issuer ID numbers have been
+      seen on this IP address.
+    - ``MINFRAUD_NETWORK_ACTIVITY`` - Suspicious activity has been seen on this
+      IP address across minFraud customers.
+
+    .. attribute:: code
+
+      This value is a machine-readable code identifying the
+      reason.
+
+      :type: str | None
+
+    .. attribute:: reason
+
+      This property provides a human-readable explanation of the
+      reason. The text may change at any time and should not be matched
+      against.
+
+      :type: str | None
+    """
+
+    code: Optional[str]
+    reason: Optional[str]
+
+    __slots__ = ()
+    _fields = {
+        "code": None,
+        "reason": None,
+    }
+
+
+def _create_ip_risk_reasons(
+    reasons: Optional[List[Dict[str, str]]]
+) -> Tuple[IPRiskReason, ...]:
+    if not reasons:
+        return ()
+    return tuple(IPRiskReason(x) for x in reasons)  # type: ignore
+
+
 class GeoIP2Location(geoip2.records.Location):
     """Location information for the IP address.
 
@@ -69,7 +126,7 @@ class GeoIP2Location(geoip2.records.Location):
 
       The date and time of the transaction in the time
       zone associated with the IP address. The value is formatted according to
-      `RFC 3339 <http://tools.ietf.org/html/rfc3339>`_. For instance, the
+      `RFC 3339 <https://tools.ietf.org/html/rfc3339>`_. For instance, the
       local time in Boston might be returned as 2015-04-27T19:17:24-04:00.
 
       :type: str | None
@@ -126,6 +183,14 @@ class IPAddress(geoip2.models.Insights):
 
       :type: float | None
 
+    .. attribute:: risk_reasons
+
+      This tuple contains :class:`.IPRiskReason` objects identifying the
+      reasons why the IP address received the associated risk. This will be
+      an empty tuple if there are no reasons.
+
+      :type: tuple[IPRiskReason]
+
     .. attribute:: city
 
       City object for the requested IP address.
@@ -189,6 +254,7 @@ class IPAddress(geoip2.models.Insights):
     country: GeoIP2Country
     location: GeoIP2Location
     risk: Optional[float]
+    risk_reasons: Tuple[IPRiskReason, ...]
 
     def __init__(self, ip_address: Dict[str, Any]) -> None:
         if ip_address is None:
@@ -200,6 +266,7 @@ class IPAddress(geoip2.models.Insights):
         self.country = GeoIP2Country(locales, **ip_address.get("country", {}))
         self.location = GeoIP2Location(**ip_address.get("location", {}))
         self.risk = ip_address.get("risk", None)
+        self.risk_reasons = _create_ip_risk_reasons(ip_address.get("risk_reasons"))
         self._finalized = True
 
     # Unfortunately the GeoIP2 models are not immutable, only the records. This
@@ -289,7 +356,7 @@ class Device:
 
     In order to receive device output from minFraud Insights or minFraud
     Factors, you must be using the `Device Tracking Add-on
-    <https://dev.maxmind.com/minfraud/device/>`_.
+    <https://dev.maxmind.com/minfraud/track-devices?lang=en>`_.
 
     .. attribute:: confidence
 
@@ -351,27 +418,38 @@ class Disposition:
     .. attribute:: action
 
       The action to take on the transaction as defined by your custom rules.
-      The current set of values are "accept", "manual_review", and "reject".
-      If you do not have custom rules set up, ``None`` will be returned.
+      The current set of values are "accept", "manual_review", "reject", and
+      "test". If you do not have custom rules set up, ``None`` will be
+      returned.
 
       :type: str | None
 
     .. attribute:: reason
 
-      The reason for the action. The current possible values are
-      "custom_rule", "block_list", and "default". If you do not have custom
-      rules set up, ``None`` will be returned.
+      The reason for the action. The current possible values are "custom_rule"
+      and "default". If you do not have custom rules set up, ``None`` will be
+      returned.
+
+      :type: str | None
+
+    .. attribute:: rule_label
+
+      The label of the custom rule that was triggered. If you do not have
+      custom rules set up, the triggered custom rule does not have a label, or
+      no custom rule was triggered, ``None`` will be returned.
 
       :type: str | None
     """
 
     action: Optional[str]
     reason: Optional[str]
+    rule_label: Optional[str]
 
     __slots__ = ()
     _fields = {
         "action": None,
         "reason": None,
+        "rule_label": None,
     }
 
 
@@ -463,7 +541,7 @@ class CreditCard:
     .. attribute:: country
 
       This property contains the `ISO 3166-1 alpha-2 country code
-      <http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2>`_ associated with the
+      <https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2>`_ associated with the
       location of the majority of customers using this credit card as
       determined by their billing address. In cases where the location of
       customers is highly mixed, this defaults to the country of the bank
@@ -691,7 +769,7 @@ class ServiceWarning:
 
       This value is a machine-readable code identifying the
       warning. See the `web service documentation
-      <https://dev.maxmind.com/minfraud/#Warning>`_
+      <https://dev.maxmind.com/minfraud/api-documentation/responses?lang=en#schema--response--warning>`_
       for the current list of of warning codes.
 
       :type: str | None
@@ -729,12 +807,12 @@ class ServiceWarning:
 def _create_warnings(warnings: List[Dict[str, str]]) -> Tuple[ServiceWarning, ...]:
     if not warnings:
         return ()
-    return tuple([ServiceWarning(x) for x in warnings])  # type: ignore
+    return tuple(ServiceWarning(x) for x in warnings)  # type: ignore
 
 
 @_inflate_to_namedtuple
 class Subscores:
-    """Subscores used in calculating the overall risk score.
+    """Risk factor scores used in calculating the overall risk score.
 
     .. attribute:: avs_result
 
@@ -785,7 +863,7 @@ class Subscores:
     .. attribute:: country_mismatch
 
       The risk associated with the combination of IP country, card issuer
-      country, billing country, and shipping country.  If present, this is a
+      country, billing country, and shipping country. If present, this is a
       value in the range 0.01 to 99.
 
       :type: float | None
@@ -834,9 +912,9 @@ class Subscores:
       :type: float | None
 
       .. deprecated:: 1.8.0
-        Deprecated effective August 29, 2019. This subscore will default to 1
-        and will be removed in a future release. The user tenure on email is
-        reflected in the email address subscore output.
+        Deprecated effective August 29, 2019. This risk factor score will
+        default to 1 and will be removed in a future release. The user
+        tenure on email is reflected in the email address risk factor score.
 
       .. seealso::
         :py:attr:`minfraud.models.Subscores.email_address`
@@ -849,9 +927,9 @@ class Subscores:
       :type: float | None
 
       .. deprecated:: 1.8.0
-        Deprecated effective August 29, 2019. This subscore will default to 1
-        and will be removed in a future release. The IP tenure is reflected in
-        the overall risk score.
+        Deprecated effective August 29, 2019. This risk factor score will
+        default to 1 and will be removed in a future release. The IP tenure
+        is reflected in the overall risk score.
 
       .. seealso::
         :py:attr:`minfraud.models.Score.risk_score`
@@ -1042,8 +1120,9 @@ class Factors:
 
     .. attribute:: subscores
 
-      A :class:`.Subscores` object containing subscores for many of the
-      individual components that are used to calculate the overall risk score.
+      A :class:`.Subscores` object containing scores for many of the
+      individual risk factors that are used to calculate the overall risk
+      score.
     """
 
     billing_address: BillingAddress
@@ -1058,7 +1137,7 @@ class Factors:
     risk_score: float
     shipping_address: ShippingAddress
     subscores: Subscores
-    warnings: List[ServiceWarning]
+    warnings: Tuple[ServiceWarning, ...]
 
     __slots__ = ()
     _fields = {
@@ -1182,7 +1261,7 @@ class Insights:
     queries_remaining: int
     risk_score: float
     shipping_address: ShippingAddress
-    warnings: List[ServiceWarning]
+    warnings: Tuple[ServiceWarning, ...]
 
     __slots__ = ()
     _fields = {
@@ -1266,7 +1345,7 @@ class Score:
     ip_address: ScoreIPAddress
     queries_remaining: int
     risk_score: float
-    warnings: List[ServiceWarning]
+    warnings: Tuple[ServiceWarning, ...]
 
     __slots__ = ()
     _fields = {
