@@ -8,6 +8,7 @@ class TestModels(unittest.TestCase):
         """This tests some level of _shallow_ immutability for these classes"""
         T = namedtuple("T", ["obj", "attr"])
         models = [
+            T(IPRiskReason(), "code"),
             T(Issuer(), "name"),
             T(CreditCard(), "country"),
             T(Device(), "id"),
@@ -22,9 +23,7 @@ class TestModels(unittest.TestCase):
         ]
         for model in models:
             for attr in (model.attr, "does_not_exist"):
-                with self.assertRaises(
-                    AttributeError, msg="{0!s} - {1}".format(model.obj, attr)
-                ):
+                with self.assertRaises(AttributeError, msg=f"{model.obj} - {attr}"):
                     setattr(model.obj, attr, 5)  # type: ignore
 
     def test_billing_address(self):
@@ -100,10 +99,13 @@ class TestModels(unittest.TestCase):
         self.assertEqual(local_time, device.local_time)
 
     def test_disposition(self):
-        disposition = Disposition({"action": "accept", "reason": "default"})
+        disposition = Disposition(
+            {"action": "accept", "reason": "default", "rule_label": "custom rule label"}
+        )
 
         self.assertEqual("accept", disposition.action)
         self.assertEqual("default", disposition.reason)
+        self.assertEqual("custom rule label", disposition.rule_label)
 
     def test_email(self):
         first_seen = "2016-01-01"
@@ -154,6 +156,16 @@ class TestModels(unittest.TestCase):
                     "local_time": time,
                 },
                 "risk": 99,
+                "risk_reasons": [
+                    {
+                        "code": "ANONYMOUS_IP",
+                        "reason": "The IP address belongs to an anonymous network. See /ip_address/traits for more details.",
+                    },
+                    {
+                        "code": "MINFRAUD_NETWORK_ACTIVITY",
+                        "reason": "Suspicious activity has been seen on this IP address across minFraud customers.",
+                    },
+                ],
                 "traits": {
                     "is_anonymous": True,
                     "is_anonymous_proxy": True,
@@ -163,6 +175,8 @@ class TestModels(unittest.TestCase):
                     "is_residential_proxy": True,
                     "is_satellite_provider": True,
                     "is_tor_exit_node": True,
+                    "mobile_country_code": "310",
+                    "mobile_network_code": "004",
                 },
             }
         )
@@ -179,8 +193,26 @@ class TestModels(unittest.TestCase):
         self.assertEqual(True, address.traits.is_residential_proxy)
         self.assertEqual(True, address.traits.is_satellite_provider)
         self.assertEqual(True, address.traits.is_tor_exit_node)
+        self.assertEqual("310", address.traits.mobile_country_code)
+        self.assertEqual("004", address.traits.mobile_network_code)
         self.assertEqual(True, address.country.is_high_risk)
 
+        self.assertEqual("ANONYMOUS_IP", address.risk_reasons[0].code)
+        self.assertEqual(
+            "The IP address belongs to an anonymous network. See /ip_address/traits for more details.",
+            address.risk_reasons[0].reason,
+        )
+
+        self.assertEqual("MINFRAUD_NETWORK_ACTIVITY", address.risk_reasons[1].code)
+        self.assertEqual(
+            "Suspicious activity has been seen on this IP address across minFraud customers.",
+            address.risk_reasons[1].reason,
+        )
+
+    def test_empty_address(self):
+        address = IPAddress({})
+        self.assertEqual((), address.risk_reasons)
+
     def test_score_ip_address(self):
         address = ScoreIPAddress({"risk": 99})
         self.assertEqual(99, address.risk)
